"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryClient = void 0;
var utils_1 = require("./utils");
var queryCache_1 = require("./queryCache");
var mutationCache_1 = require("./mutationCache");
var focusManager_1 = require("./focusManager");
var onlineManager_1 = require("./onlineManager");
var notifyManager_1 = require("./notifyManager");
var infiniteQueryBehavior_1 = require("./infiniteQueryBehavior");
// CLASS
var QueryClient = /** @class */ (function () {
    function QueryClient(config) {
        if (config === void 0) { config = {}; }
        this.queryCache = config.queryCache || new queryCache_1.QueryCache();
        this.mutationCache = config.mutationCache || new mutationCache_1.MutationCache();
        this.defaultOptions = config.defaultOptions || {};
        this.queryDefaults = [];
        this.mutationDefaults = [];
    }
    // 挂载
    QueryClient.prototype.mount = function () {
        var _this = this;
        this.unsubscribeFocus = focusManager_1.focusManager.subscribe(function () {
            if (focusManager_1.focusManager.isFocused() && onlineManager_1.onlineManager.isOnline()) {
                _this.mutationCache.onFocus();
                _this.queryCache.onFocus();
            }
        });
        this.unsubscribeOnline = onlineManager_1.onlineManager.subscribe(function () {
            if (focusManager_1.focusManager.isFocused() && onlineManager_1.onlineManager.isOnline()) {
                _this.mutationCache.onOnline();
                _this.queryCache.onOnline();
            }
        });
    };
    // 卸载
    QueryClient.prototype.unmount = function () {
        var _a, _b;
        (_a = this.unsubscribeFocus) === null || _a === void 0 ? void 0 : _a.call(this);
        (_b = this.unsubscribeOnline) === null || _b === void 0 ? void 0 : _b.call(this);
    };
    QueryClient.prototype.isFetching = function (arg1, arg2) {
        var filters = utils_1.parseFilterArgs(arg1, arg2)[0];
        filters.fetching = true;
        return this.queryCache.findAll(filters).length;
    };
    QueryClient.prototype.isMutating = function (filters) {
        return this.mutationCache.findAll(__assign(__assign({}, filters), { fetching: true })).length;
    };
    QueryClient.prototype.getQueryData = function (queryKey, filters) {
        var _a;
        return (_a = this.queryCache.find(queryKey, filters)) === null || _a === void 0 ? void 0 : _a.state.data;
    };
    QueryClient.prototype.setQueryData = function (queryKey, updater, options) {
        var parsedOptions = utils_1.parseQueryArgs(queryKey);
        var defaultedOptions = this.defaultQueryOptions(parsedOptions);
        return this.queryCache
            .build(this, defaultedOptions)
            .setData(updater, options);
    };
    QueryClient.prototype.setQueriesData = function (queryKeyOrFilters, updater, options) {
        var _this = this;
        return notifyManager_1.notifyManager.batch(function () {
            return _this.getQueryCache()
                .findAll(queryKeyOrFilters)
                .map(function (_a) {
                var queryKey = _a.queryKey;
                return [
                    queryKey,
                    _this.setQueryData(queryKey, updater, options),
                ];
            });
        });
    };
    QueryClient.prototype.getQueryState = function (queryKey, filters) {
        var _a;
        return (_a = this.queryCache.find(queryKey, filters)) === null || _a === void 0 ? void 0 : _a.state;
    };
    QueryClient.prototype.removeQueries = function (arg1, arg2) {
        var filters = utils_1.parseFilterArgs(arg1, arg2)[0];
        var queryCache = this.queryCache;
        notifyManager_1.notifyManager.batch(function () {
            queryCache.findAll(filters).forEach(function (query) {
                queryCache.remove(query);
            });
        });
    };
    QueryClient.prototype.resetQueries = function (arg1, arg2, arg3) {
        var _this = this;
        var _a = utils_1.parseFilterArgs(arg1, arg2, arg3), filters = _a[0], options = _a[1];
        var queryCache = this.queryCache;
        var refetchFilters = __assign(__assign({}, filters), { active: true });
        return notifyManager_1.notifyManager.batch(function () {
            queryCache.findAll(filters).forEach(function (query) {
                query.reset();
            });
            return _this.refetchQueries(refetchFilters, options);
        });
    };
    QueryClient.prototype.cancelQueries = function (arg1, arg2, arg3) {
        var _this = this;
        var _a = utils_1.parseFilterArgs(arg1, arg2, arg3), filters = _a[0], _b = _a[1], cancelOptions = _b === void 0 ? {} : _b;
        if (typeof cancelOptions.revert === 'undefined') {
            cancelOptions.revert = true;
        }
        var promises = notifyManager_1.notifyManager.batch(function () {
            return _this.queryCache.findAll(filters).map(function (query) { return query.cancel(cancelOptions); });
        });
        return Promise.all(promises).then(utils_1.noop).catch(utils_1.noop);
    };
    QueryClient.prototype.invalidateQueries = function (arg1, arg2, arg3) {
        var _this = this;
        var _a, _b, _c;
        var _d = utils_1.parseFilterArgs(arg1, arg2, arg3), filters = _d[0], options = _d[1];
        var refetchFilters = __assign(__assign({}, filters), { 
            // if filters.refetchActive is not provided and filters.active is explicitly false,
            // e.g. invalidateQueries({ active: false }), we don't want to refetch active queries
            active: (_b = (_a = filters.refetchActive) !== null && _a !== void 0 ? _a : filters.active) !== null && _b !== void 0 ? _b : true, inactive: (_c = filters.refetchInactive) !== null && _c !== void 0 ? _c : false });
        return notifyManager_1.notifyManager.batch(function () {
            _this.queryCache.findAll(filters).forEach(function (query) {
                query.invalidate();
            });
            return _this.refetchQueries(refetchFilters, options);
        });
    };
    QueryClient.prototype.refetchQueries = function (arg1, arg2, arg3) {
        var _this = this;
        var _a = utils_1.parseFilterArgs(arg1, arg2, arg3), filters = _a[0], options = _a[1];
        var promises = notifyManager_1.notifyManager.batch(function () {
            return _this.queryCache.findAll(filters).map(function (query) { return query.fetch(); });
        });
        var promise = Promise.all(promises).then(utils_1.noop);
        if (!(options === null || options === void 0 ? void 0 : options.throwOnError)) {
            promise = promise.catch(utils_1.noop);
        }
        return promise;
    };
    QueryClient.prototype.fetchQuery = function (arg1, arg2, arg3) {
        var parsedOptions = utils_1.parseQueryArgs(arg1, arg2, arg3);
        var defaultedOptions = this.defaultQueryOptions(parsedOptions);
        // https://github.com/tannerlinsley/react-query/issues/652
        if (typeof defaultedOptions.retry === 'undefined') {
            defaultedOptions.retry = false;
        }
        var query = this.queryCache.build(this, defaultedOptions);
        return query.isStaleByTime(defaultedOptions.staleTime)
            ? query.fetch(defaultedOptions)
            : Promise.resolve(query.state.data);
    };
    QueryClient.prototype.prefetchQuery = function (arg1, arg2, arg3) {
        return this.fetchQuery(arg1, arg2, arg3)
            .then(utils_1.noop)
            .catch(utils_1.noop);
    };
    QueryClient.prototype.fetchInfiniteQuery = function (arg1, arg2, arg3) {
        var parsedOptions = utils_1.parseQueryArgs(arg1, arg2, arg3);
        parsedOptions.behavior = infiniteQueryBehavior_1.infiniteQueryBehavior();
        return this.fetchQuery(parsedOptions);
    };
    QueryClient.prototype.prefetchInfiniteQuery = function (arg1, arg2, arg3) {
        return this.fetchInfiniteQuery(arg1, arg2, arg3)
            .then(utils_1.noop)
            .catch(utils_1.noop);
    };
    QueryClient.prototype.cancelMutations = function () {
        var _this = this;
        var promises = notifyManager_1.notifyManager.batch(function () {
            return _this.mutationCache.getAll().map(function (mutation) { return mutation.cancel(); });
        });
        return Promise.all(promises).then(utils_1.noop).catch(utils_1.noop);
    };
    QueryClient.prototype.resumePausedMutations = function () {
        return this.getMutationCache().resumePausedMutations();
    };
    QueryClient.prototype.executeMutation = function (options) {
        return this.mutationCache.build(this, options).execute();
    };
    QueryClient.prototype.getQueryCache = function () {
        return this.queryCache;
    };
    QueryClient.prototype.getMutationCache = function () {
        return this.mutationCache;
    };
    QueryClient.prototype.getDefaultOptions = function () {
        return this.defaultOptions;
    };
    QueryClient.prototype.setDefaultOptions = function (options) {
        this.defaultOptions = options;
    };
    QueryClient.prototype.setQueryDefaults = function (queryKey, options) {
        var result = this.queryDefaults.find(function (x) { return utils_1.hashQueryKey(queryKey) === utils_1.hashQueryKey(x.queryKey); });
        if (result) {
            result.defaultOptions = options;
        }
        else {
            this.queryDefaults.push({ queryKey: queryKey, defaultOptions: options });
        }
    };
    QueryClient.prototype.getQueryDefaults = function (queryKey) {
        var _a;
        return queryKey
            ? (_a = this.queryDefaults.find(function (x) { return utils_1.partialMatchKey(queryKey, x.queryKey); })) === null || _a === void 0 ? void 0 : _a.defaultOptions
            : undefined;
    };
    QueryClient.prototype.setMutationDefaults = function (mutationKey, options) {
        var result = this.mutationDefaults.find(function (x) { return utils_1.hashQueryKey(mutationKey) === utils_1.hashQueryKey(x.mutationKey); });
        if (result) {
            result.defaultOptions = options;
        }
        else {
            this.mutationDefaults.push({ mutationKey: mutationKey, defaultOptions: options });
        }
    };
    QueryClient.prototype.getMutationDefaults = function (mutationKey) {
        var _a;
        return mutationKey
            ? (_a = this.mutationDefaults.find(function (x) {
                return utils_1.partialMatchKey(mutationKey, x.mutationKey);
            })) === null || _a === void 0 ? void 0 : _a.defaultOptions
            : undefined;
    };
    QueryClient.prototype.defaultQueryOptions = function (options) {
        if (options === null || options === void 0 ? void 0 : options._defaulted) {
            return options;
        }
        var defaultedOptions = __assign(__assign(__assign(__assign({}, this.defaultOptions.queries), this.getQueryDefaults(options === null || options === void 0 ? void 0 : options.queryKey)), options), { _defaulted: true });
        if (!defaultedOptions.queryHash && defaultedOptions.queryKey) {
            defaultedOptions.queryHash = utils_1.hashQueryKeyByOptions(defaultedOptions.queryKey, defaultedOptions);
        }
        return defaultedOptions;
    };
    QueryClient.prototype.defaultQueryObserverOptions = function (options) {
        return this.defaultQueryOptions(options);
    };
    QueryClient.prototype.defaultMutationOptions = function (options) {
        if (options === null || options === void 0 ? void 0 : options._defaulted) {
            return options;
        }
        return __assign(__assign(__assign(__assign({}, this.defaultOptions.mutations), this.getMutationDefaults(options === null || options === void 0 ? void 0 : options.mutationKey)), options), { _defaulted: true });
    };
    QueryClient.prototype.clear = function () {
        this.queryCache.clear();
        this.mutationCache.clear();
    };
    return QueryClient;
}());
exports.QueryClient = QueryClient;
