"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useBaseQuery = void 0;
var vue_1 = require("vue");
var QueryClientProvider_1 = require("./QueryClientProvider");
function genArr(data) {
    return Array.isArray(data) ? data : data == undefined ? [] : [data];
}
function useBaseQuery(options, Observer) {
    var queryClient = QueryClientProvider_1.useQueryClient();
    var defaultedOptions = queryClient.defaultQueryObserverOptions(options);
    var observer = new Observer(queryClient, defaultedOptions);
    var result = vue_1.reactive(observer.getCurrentResult());
    // 监听依赖项的变化
    vue_1.watch(function () { return options.queryKey; }, function () {
        observer.setOptions(options); // 主要里面存在一个updateQuery方法
        observer.updateResult();
    }, {
        deep: true,
    });
    // 订阅数据的变化
    var unsubscribe;
    vue_1.onMounted(function () {
        // Update result to make sure we did not miss any query updates
        // between creating the observer and subscribing to it.
        observer.updateResult();
        unsubscribe = observer.subscribe(function (res) {
            Object.assign(result, res);
        });
    });
    vue_1.onUnmounted(function () {
        unsubscribe === null || unsubscribe === void 0 ? void 0 : unsubscribe();
    });
    return result;
}
exports.useBaseQuery = useBaseQuery;
