"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useIsMutating = void 0;
var vue_1 = require("vue");
var notifyManager_1 = require("../core/notifyManager");
var QueryClientProvider_1 = require("./QueryClientProvider");
/**
 * 根据筛选获取当前突变的数量
 * @param filters 筛选条件
 * @returns 突变的数量
 */
function useIsMutating(filters) {
    var queryClient = QueryClientProvider_1.useQueryClient();
    var isMutating = vue_1.ref(queryClient.isMutating(filters));
    // 订阅观察者
    var unsubscribe;
    vue_1.onMounted(function () {
        unsubscribe = queryClient.getMutationCache().subscribe(function () {
            notifyManager_1.notifyManager.batchCalls(function () {
                var newIsMutating = queryClient.isMutating(filters);
                if (isMutating.value !== newIsMutating) {
                    isMutating.value = newIsMutating;
                }
            })();
        });
    });
    vue_1.onUnmounted(function () {
        unsubscribe === null || unsubscribe === void 0 ? void 0 : unsubscribe();
    });
    return isMutating;
}
exports.useIsMutating = useIsMutating;
