"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useQueries = void 0;
var vue_1 = require("vue");
var queriesObserver_1 = require("../core/queriesObserver");
var QueryClientProvider_1 = require("./QueryClientProvider");
/**
 * 批量使用useQuery
 */
function useQueries(queries) {
    var queryClient = QueryClientProvider_1.useQueryClient();
    var defaultedQueries = queries.map(function (options) {
        var defaultedOptions = queryClient.defaultQueryObserverOptions(options);
        // Make sure the results are already in fetching state before subscribing or updating options
        defaultedOptions.optimisticResults = true;
        return defaultedOptions;
    });
    var observer = new queriesObserver_1.QueriesObserver(queryClient, defaultedQueries);
    var result = vue_1.reactive(observer.getOptimisticResult(defaultedQueries));
    // 订阅数据的变化
    var unsubscribe;
    vue_1.onMounted(function () {
        unsubscribe = observer.subscribe(function (res) {
            Object.assign(result, res);
        });
    });
    vue_1.onUnmounted(function () {
        unsubscribe === null || unsubscribe === void 0 ? void 0 : unsubscribe();
    });
    vue_1.watchEffect(function () {
        // Do not notify on updates because of changes in the options because
        // these changes should already be reflected in the optimistic result.
        observer.setQueries(defaultedQueries, { listeners: false });
    });
    return result;
}
exports.useQueries = useQueries;
